package com.sun.media.sound;

import javax.sound.midi.Sequence;
import java.io.FileInputStream;
import java.io.File;
import java.io.BufferedInputStream;
import java.net.URL;
import java.io.DataInputStream;
import java.io.IOException;
import javax.sound.midi.InvalidMidiDataException;
import javax.sound.midi.MidiFileFormat;
import java.io.InputStream;
import javax.sound.midi.spi.MidiFileReader;

public final class StandardMidiFileReader extends MidiFileReader
{
    private static final int MThd_MAGIC = 1297377380;
    private static final int bisBufferSize = 1024;
    
    public StandardMidiFileReader() {
    }
    
    @Override
    public MidiFileFormat getMidiFileFormat(InputStream inputStream) throws InvalidMidiDataException, IOException {
        return this.getMidiFileFormatFromStream(inputStream, -1, null);
    }
    
    private MidiFileFormat getMidiFileFormatFromStream(InputStream in, int bytes, SMFParser smfParser) throws InvalidMidiDataException, IOException {
        int readlimit = 16;
        int n = -1;
        DataInputStream stream;
        if ((in instanceof DataInputStream)) {
            stream = (DataInputStream)in;
        }
        else {
            stream = new DataInputStream(in);
        }
        if (smfParser == null) {
            stream.mark(readlimit);
        }
        else {
            smfParser.stream = stream;
        }
        int short1;
        int short3;
        float divisionType;
        try {
            if (stream.readInt() != 1297377380) {
                throw new InvalidMidiDataException("not a valid MIDI file");
            }
            int n2 = stream.readInt() - 6;
            short1 = stream.readShort();
            int short2 = stream.readShort();
            short3 = stream.readShort();
            if (short3 > 0) {
                divisionType = 0.0f;
            }
            else {
                int i = -1 * (short3 >> 8);
                switch (i) {
                    case 24: {
                        divisionType = 24.0f;
                        break;
                    }
                    case 25: {
                        divisionType = 25.0f;
                        break;
                    }
                    case 29: {
                        divisionType = 29.97f;
                        break;
                    }
                    case 30: {
                        divisionType = 30.0f;
                        break;
                    }
                    default: {
                        throw new InvalidMidiDataException("Unknown frame code: " + i);
                    }
                }
                short3 &= 0xFF;
            }
            if (smfParser != null) {
                stream.skip((long)n2);
                smfParser.tracks = short2;
            }
        }
        finally {
            if (smfParser == null) {
                stream.reset();
            }
        }
        return new MidiFileFormat(short1, divisionType, short3, bytes, (long)n);
    }
    
    @Override
    public MidiFileFormat getMidiFileFormat(URL url) throws InvalidMidiDataException, IOException {
        BufferedInputStream bufferedInputStream = new BufferedInputStream(url.openStream(), 1024);
        MidiFileFormat midiFileFormat = null;
        try {
            midiFileFormat = this.getMidiFileFormat(bufferedInputStream);
        }
        finally {
            bufferedInputStream.close();
        }
        return midiFileFormat;
    }
    
    @Override
    public MidiFileFormat getMidiFileFormat(File file) throws InvalidMidiDataException, IOException {
        BufferedInputStream bufferedInputStream = new BufferedInputStream(new FileInputStream(file), 1024);
        long length = file.length();
        if (length > 2147483647L) {
            length = -1L;
        }
        MidiFileFormat midiFileFormatFromStream = null;
        try {
            midiFileFormatFromStream = this.getMidiFileFormatFromStream(bufferedInputStream, (int)length, null);
        }
        finally {
            bufferedInputStream.close();
        }
        return midiFileFormatFromStream;
    }
    
    @Override
    public Sequence getSequence(InputStream inputStream) throws InvalidMidiDataException, IOException {
        SMFParser smfParser = new SMFParser();
        MidiFileFormat midiFileFormatFromStream = this.getMidiFileFormatFromStream(inputStream, -1, smfParser);
        if (midiFileFormatFromStream.getType() != 0 && midiFileFormatFromStream.getType() != 1) {
            throw new InvalidMidiDataException("Invalid or unsupported file type: " + midiFileFormatFromStream.getType());
        }
        Sequence sequence = new Sequence(midiFileFormatFromStream.getDivisionType(), midiFileFormatFromStream.getResolution());
        for (int n = 0; n < smfParser.tracks && smfParser.nextTrack(); ++n) {
            smfParser.readTrack(sequence.createTrack());
        }
        return sequence;
    }
    
    @Override
    public Sequence getSequence(URL url) throws InvalidMidiDataException, IOException {
        BufferedInputStream bufferedInputStream = new BufferedInputStream(url.openStream(), 1024);
        Sequence sequence = null;
        try {
            sequence = this.getSequence(bufferedInputStream);
        }
        finally {
            bufferedInputStream.close();
        }
        return sequence;
    }
    
    @Override
    public Sequence getSequence(File file) throws InvalidMidiDataException, IOException {
        BufferedInputStream bufferedInputStream = new BufferedInputStream(new FileInputStream(file), 1024);
        Sequence sequence = null;
        try {
            sequence = this.getSequence(bufferedInputStream);
        }
        finally {
            bufferedInputStream.close();
        }
        return sequence;
    }
}
